
##
##  covgscmodel.r                                                                  R. Schlittgen 24.06.2018
##
#' Modification of a covariance matrix of a cb sem model to allow for correlated errors in the regression equaton of loadings.
#'
#' \code{gscmcovce} determines the covariance matrix of a GSC model with correlated errors in the regression equaton of loadings.  
#' @param B   (q,q) lower triangular matrix describing the interrelations of the latent variables:
#'             b_ij = 1 regression coefficient of eta_j in the regression relation in which eta_i is the depend variable
#'             b_ij = 0 if eta_i does not depend on eta_j in a direct way  (b_ii = 0 !)
#' @param  indicatorx vector describing with which exogenous composite the X-variables are connected
#' @param  indicatory vector describing with which endogenous composite the Y-variables are connected
#' @param  lambdax  vector of loadings of indicators for exogenous composites or NULL when there are no loadings for the X-variables in the model
#' @param  lambday  vector of loadings of indicators for endogenous composites 
#' @param  wx  vector of weights for building exogenous composites or NULL when  there are no loadings for the X-variables in the model
#' @param  wy  vector of weights for building endogenous composites 
#' @param  S covariance matrix ofindicators
#' @param  Scomp covariance matrix of composites
#' @return out list with components
#'        \tabular{ll}{ S \tab covariance matrix of manifest variables   \cr 
#'          Sdd \tab diagonal matrix of variances of errors of X variable loadings or NA  \cr
#'          See \tab  diagonal matrix of variances of errors of Y variable loadings or NA \cr
#'          optval  \tab vector with values of optimisation criterion
#'         }
#' @examples
#' Sxixi <- matrix(c(1.0,  0.01,  0.01, 1),2,2)   
#' B <- matrix(c( 0,0,0, 0,0,0, 0.7,0.4,0),3,3,byrow=TRUE) 
#' indicatorx <- c(1,1,1,2,2,2)
#' indicatory <- c(1,1,1)                     
#' lambdax <- c(0.83,0.87,0.87,0.91,0.88,0.82)
#' lambday <- c(0.89,0.90,0.80)  
#' out1 <- gscmcov(B,indicatorx,indicatory,lambdax=lambdax,lambday=lambday,
#'                 wx=NULL,wy=NULL,Sxixi,R2=NULL)
#' out1a <- Checkw(B,indicatorx,indicatory,lambdax=lambdax,lambday=lambday,
#'                 wx=FALSE,wy=FALSE,Sxixi,R2=NULL) 
#' out2 <- gscmcovce(B,indicatorx,indicatory,lambdax=lambdax,lambday,
#'                   wx=out1a$wx,out1a$wy,out1$S,out1$Scomp) 
#'
#' @export
gscmcovce <- function(B,indicatorx,indicatory,lambdax=NULL,lambday,wx=NULL,wy,S,Scomp){    
value1 <- NA
value2 <- NA
Sdd <- NA

Lambdax <- NULL
Lambday <- NULL
Wx <- NULL     
Wy <- NULL     
 
q <- nrow(B)                                      # number of composites
q1 <- length(unique(indicatorx))         # number of exogeneous composites 
q2 <- length(unique(indicatory))         # number of endogeneous composites
p1 <- length(indicatorx)                     #  number of indicators of exogeneous composites 
p2 <- length(indicatory)                     # number of indicators of endogeneous composites
p <- p1 + p2                                     # number of manifest variables    
  
if( is.numeric( lambdax) ){
                                                               # construction of Lx
   Lx <- matrix(indicatorx,p1,q1) 
   Lx <- 1*(Lx == matrix(c(1:q1),p1,q1,byrow=T))   
      p11 <- colSums(Lx)                            # preparation for correlation of indicators
      P11 <- p11*(p11-1)/2                        # preparation for correlation of indicators  
   Lx <- Lx*lambdax   
   Sxixi <- Scomp[1:q1,1:q1,drop=F]
   Sxx <- Lx%*%Sxixi%*%t(Lx)  
   Sdd <- diag(1-diag(Sxx))  
   
   #  determine correlations for blocks of delta  
    
   sstart <- rep(0,sum(P11)) 
   out1  <- optim(sstart, checkwce , gr = NULL, indicatorx,wx,Lx,Sxixi, method="BFGS" )  
   value1 <- out1$value     
 
   # determine Sdd with out1$par
   
   start <- c(0,cumsum(p11))
   start1 <- c(0,cumsum(P11)) 
   s <- out1$par 
   Sdd <- matrix(0,p1,p1)
   for (i in c(1:q1)){  
          teil <- Sdd[(start[i]+1):start[i+1],(start[i]+1):start[i+1]]
          teil[ upper.tri(teil) ] <- s[(start1[i]+1):start1[i+1]]
          Sdd[(start[i]+1):start[i+1],(start[i]+1):start[i+1]] <- teil
          }  
    Sdd <- Sdd + t(Sdd) 
    diag(Sdd) <- 1 - diag(Sxx)  
    
    #  reconstruction of  Sxx
    
    Sxx <- Sxx + Sdd
    S[1:p1,1:p1] <- Sxx 
}   
                                                               # construction of Ly
   Ly <- matrix(indicatory,p2,q2)
   Ly <- 1*(Ly == matrix(c(1:q2),p2,q2,byrow=T)) 
     p22 <- colSums(Ly)                            # preparation for correlation of indicators 
     P22 <- p22*(p22-1)/2                        # preparation for correlation of indicators   
   Ly <- Ly*lambday      

   Setaeta <- Scomp[(q1+1):(q1+q2),(q1+1):(q1+q2),drop=F]

   Syy <- Ly%*%Setaeta%*%t(Ly) 
   See <- diag(1 - diag(Syy))  
    
   # determine correlations for blocks of epsilon
  
   sstart <- rep(0,sum(P22)) 
   out2  <- optim(sstart, checkwce , gr = NULL,indicatory,wy,Ly,Setaeta, method="BFGS" ) 
   value2 <- out2$value
   
   # construction of See 
   
   start <- c(0,cumsum(p22))
   start1 <- c(0,cumsum(P22)) 
   s <- out2$par 
   See <- matrix(0,p2,p2)
   for (i in c(1:q2)){  
          teil <- See[(start[i]+1):start[i+1],(start[i]+1):start[i+1]]
          teil[ upper.tri(teil) ] <- s[(start1[i]+1):start1[i+1]]
          See[(start[i]+1):start[i+1],(start[i]+1):start[i+1]] <- teil
          }  
    See <- See + t(See)
    diag(See) <- 1 - diag(Syy)   
 
    #  reconstruction of  Syy
        
    Syy <- Syy + See 
    S[(p1+1):p,(p1+1):p] <- Syy

################        

out <- list(S=S,Sdd=Sdd,See=See,optval=c(value1,value2))
return(out)
}

#' \code{checkwce} compares two formulations of the covariance matrix of composites. For use in   gscmcovce
#' @param s   vector of correlations of errors in the regression relation of loadings 
#' @param  indicator vector describing with which composite the indicators are connected 
#' @param  w  vector of weights for building  composites  
#' @param  L matrix of loadings
#' @param  Scomp covariance matrix of  composites 
#' @return out sum of squared differences of two formulations of the covariance matrix of composites
#'
#' @export
checkwce <- function(s,indicator,w,L,Scomp){  
   p <- length(indicator)
   q <- length(unique(indicator)) 
   W <- matrix(indicator,p,q)      # construction of W1 
   W <- 1*(W == matrix(c(1:q),p,q,byrow=T))   
   p11 <- colSums(W)
   P11 <- p11*(p11-1)/2  
   start <- c(0,cumsum(p11))
   start1 <- c(0,cumsum(P11))
   W <- W*w 
   Sdd <- matrix(0,p,p)
   for (i in c(1:q)){  
          teil <- Sdd[(start[i]+1):start[i+1],(start[i]+1):start[i+1]]
          teil[ upper.tri(teil) ] <- s[(start1[i]+1):start1[i+1]]
          Sdd[(start[i]+1):start[i+1],(start[i]+1):start[i+1]] <- teil
          }  
    Sdd <- Sdd + t(Sdd)
    diag(Sdd) <- 1 - diag(L%*%Scomp%*%t(L)  ) 
  out <- sum((t(W)%*%(L%*%Scomp%*%t(L) +Sdd)%*%W - Scomp)^2)  
  return(out)  
   }
 